% Images taken from:
% Olmos, A. and Kingdom, F. A. A. (2004) McGill Calibrated Colour Image
% Database, http://pirsquared.org/research/mcgilldb/

clear
close all

%%%%%%%%%%%%%%%%%%%%%%%%%
% Patch size
% p = 16; 
% q = 16;
p = 8; q = 8;
T = p*q;


current_dir = pwd;
images_dir = './images/training';
cd(images_dir)

% Read images
S{1} = imread('samplemerry0135.jpg');
S{2} = imread('sampleMerry_0030_Lasalle.jpg');
S{3} = imread('samplemerry_florida0034.jpg');
S{4} = imread('samplemerry_mexico0123.jpg');
S{5} = imread('samplepippin0097.jpg');
S{6} = imread('samplepippin_jtalon0036.jpg');
nr_images = 6;

% nr_images = 22;
% S = cell(1, nr_images);
% 
% S{1} = imread('sampleMerry_0007_Lasalle.jpg');
% S{2} = imread('sampleMerry_0030_Lasalle.jpg');
% S{3} = imread('sampleMerry_0039_Lasalle.jpg');
% S{4} = imread('samplemerry_florida0004.jpg');
% S{5} = imread('samplemerry_florida0034.jpg');
% S{6} = imread('samplemerry_mexico0063.jpg');
% S{7} = imread('samplemerry_mexico0093.jpg');
% S{8} = imread('samplemerry_mexico0123.jpg');
% S{9} = imread('samplemerry_mtl07_019.jpg');
% S{10} = imread('samplemerry0135.jpg');
% S{11} = imread('samplemerry0196.jpg');
% S{12} = imread('samplepippin_jtalon0002.jpg');
% S{13} = imread('samplepippin_Mex07_011.jpg');
% S{14} = imread('samplepippin_ParcSafari005.jpg');
% S{15} = imread('samplepippin_Peel062.jpg');
% S{16} = imread('samplepippin_UniStrt020.jpg');
% S{17} = imread('samplepippin0021.jpg');
% S{18} = imread('samplepippin0097.jpg');
% S{19} = imread('samplepippin0223.jpg');
% S{20} = imread('samplepippin0228.jpg');
% S{21} = imread('samplepippin0229.jpg');
% S{22} = imread('samplepippin0289.jpg');

cd(current_dir);

% Convert to grayscale
for i = 1:nr_images
    S{i} = rgb2gray(S{i});
end

cd(current_dir)


% Generate patches
num = 5000;
total_nr = nr_images * num;
Sp = zeros(T, total_nr);
patch_tol = 100;

h = waitbar(0, 'Generating patches...');
ind = 1;
iter = 0;
for i = 1:nr_images
    [m n]= size(S{i});
    for j = 1:num        
        iter = iter+1;
        
        % Generate random indexes
        ind1 = uint16(ceil((m-p+1) * rand));
        ind2 = uint16(ceil((n-q+1) * rand));
        patch = S{i}(ind1:ind1+p-1, ind2:ind2+q-1);
        if var(double(patch(:))) < patch_tol
            continue
        end
        
        Sp(:, ind) = reshape(patch, T , 1);        
        ind = ind+1;
        
        waitbar(iter / total_nr, h);
    end
end
close(h)

Sp = Sp(:, 1:ind-1);
Sp = double(Sp);

% Removing means
Sp = Sp - repmat(mean(Sp), size(Sp, 1), 1);



% Overcompleteness factor
overc_fact = 1;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% ICA
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

numOfIC = overc_fact * T;
% g = 'tanh';
g = 'gauss';
FastICA_approach = 'defl';
a1 = 5;
% a2 = 1;
% decorr_fact = 0.3;
tic
[D, ~] = fastica(Sp, 'numOfIC', numOfIC, 'g', g, ...
    'approach', FastICA_approach, 'a1', a1, 'a2', a2);%, ...
%     'decorr', decorr_fact);
ICA_time = toc




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% K-SVD alg.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% % Fast K-SVD
% params.data = Sp;
% params.dictsize = overc_fact * size(Sp, 1);
% params.Tdata = 4;
% % params.exact = 1;
% params.iternum = 100;
% 
% tic
% [D, ~] = ksvd(params);
% ksvd_time = toc


dict_file = ['dictionary' num2str(p) 'x' num2str(q)];

save(dict_file, 'D')


% Show learned atoms/basis vectors?
show_basis_vectors(D, p, q, 'size', [p overc_fact * q]);

