% Inpainting test images

clear
close all


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% MC-GILL dataset
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

current_dir = pwd;

images_dir = './images/test';

cd(images_dir);

S_test{1} = imread('samplemerry0124.jpg');
S_test{2} = imread('samplemerry_florida0007.jpg');
S_test{3} = imread('samplemerry_italy0132.jpg');
S_test{4} = imread('samplepippin0094.jpg');
S_test{5} = imread('samplepippin0275.jpg');
S_test{6} = imread('samplepippin_jtalon0040.jpg');

cd(current_dir)

[m n] = size(S_test{1});
N = m*n;

% Patch size
% p = 16; q = 16;
p = 8; q = 8;
T = p*q;



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Load basis (dictionary)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

switch p
    case 16
        dict_file = 'dictionary16x16.mat';
    case 8
        dict_file = 'dictionary8x8.mat';
end

load(dict_file)




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% ALGORITHM for sparse recovery
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% alg = 'sl0';
alg = 'robustsl0';

% Parameters
switch p
    case 8
        sigmaMin = 50;
    case 16
        sigmaMin = 300;
end







%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PARAMETERS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
switch p
    case 16
        overlap = 8; 
    case 8
        overlap = 4;
end

aux_cols = q - overlap;
aux_rows = p - overlap;
nr1 = round(floor((m-p)/aux_rows));
nr2 = round(floor((n-q)/aux_cols));

firstIndices = zeros(1, nr1+2);
for i = 1:nr1+1
    firstIndices(i) = (i-1)*aux_rows + 1;
end
firstIndices(nr1+2)= m-p+1;
l1 = length(firstIndices);

secondIndices = zeros(1, nr2+2);
for j = 1:nr2+1
    secondIndices(j) = (j-1)*aux_cols + 1;
end
secondIndices(nr2+2)= n-q+1;
l2 = length(secondIndices);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%








%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Main loop
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

test_nr = 1;
for test = 1:test_nr
    
        
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Generate mask
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    udio = 0.2;
    pom = floor(udio*N);   
    restr_ind = randperm(N);
    restr = restr_ind(1:pom)';
    restr_left = restr_ind(pom+1:end)';    
    Pmat = false(N, 1);
    Pmat(restr) = true;
    Pmat = reshape(Pmat, [m n]);    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    
    
        
    
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Subsampling matrix (the special case of random subsampling)    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     s_fact = 2;
%     ms = m / s_fact;
%     ns = n / s_fact;
%     odd_ind_i = (mod(0:m-1, s_fact) == 0);
%     odd_ind_j = (mod(0:n-1, s_fact) == 0);
%     Pmat = false(m, n);
%     Pmat(odd_ind_i, odd_ind_j) = true;            
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
        

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Process all or some images?
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    for im_nr = 1:1
    
     
    S = S_test{im_nr};         
        
    
    % Show current image
    figure(1)
    imshow(S)
    title('Original image')
    
   
    S = double(S);

  
    % For denoising
%     S_noise = S;
%     S_noise(~Pmat) = 255;
    
    % Threshold very large (and very small) values
%     S_noise(S_noise > 255) = 255;
%     S_noise(S_noise < 0) = 0;
    
    % Show noisy image
%     figure(10)
%     imshow(uint8(S_noise))
%     title('Noisy image')
    
    
%     % Median filter?
%     S_median = rmedian(S_noise);
%     
%     figure(998)
%     imshow(uint8(S_median))
%     title('(Relaxed) median filter - denoised image')
    
    


    
    
   
       

    % Image with missing pixels
    Sm = S .* double(Pmat);

    
    
    
    % Show image with missing pixels
    figure(2)
    imshow(uint8(Sm))
    title('Damaged image')


    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% MYRIAD FILTERING
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Central weight
% W_myriad = 100;
% % Window size
% N_myriad = 5; 
% G = myriad_main(Sred, W_myriad, N_myriad);
% 
% figure(999)
% imshow(uint8(G))
% title('Weighted myriad filter - denoised image')
% 
% [myriad_ssim, ~] = ssim_index(S, G)
% myriad_psnr = psnr(S, G, 255)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%





    
    
    
    
   % For denoising
    % Select 'clean' pixels 
    
%     restr = reshape(Sred < 255, N, 1);
%     % Update mask
%     Pmat = false(N, 1);
%     Pmat(restr) = 1;
%     Pmat = reshape(Pmat, m, n);
    

    

    
 


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

inp_text = ['Test ' num2str(test) ' of ' num2str(test_nr) ', image ' ...
    num2str(im_nr) '...'];
h = waitbar(0, inp_text);


patch_nr = 0;
iter1 = 0;



    % Initialize reconstruction
    Srecon = zeros(m, n);
    
    Mat_average = zeros(m, n);

tic

for ind1 = firstIndices
    
    iter1 = iter1+1;
    iter2 = 0;
    
    for ind2 = secondIndices
        
        iter2 = iter2+1;
        
        
        
               
        
        patch = S(ind1:ind1+p-1, ind2:ind2+q-1);
        
        patch_mask = Pmat(ind1:ind1+p-1, ind2:ind2+q-1);
        b = reshape(patch, T, 1);        
        bmask = reshape(patch_mask, T, 1);      
        
        Proj = eye(T);
        Proj = Proj(bmask, :);
        
        
        % FOR SUPERRESOLUTION
        % Indexes of inner pixels (not affected by edges)
%         bmask = bmask & RelInd(:);
        
        patch_nr = patch_nr+1;                 

        
        b = b(bmask);    

        % Try to recover only those blocks that contain at least 0.2*T
        % pixels (we also skip 'full' blocks)
%         if l < 0.2*T || l == T 
%             continue;
%         end
        % b = P*b;
                
                    
        bmean = mean(b);
        b = b - bmean;
            
        
            


        switch lower(alg)
            
            case 'sl0'
                        
                
                sigma_decrease_factor = 0.5;
                x = SL0(D(bmask, :), b, sigmaMin, ...
                    sigma_decrease_factor);  
                
                
            case 'robustsl0'
                
                sigmaNoise = 0.1;
                x = robust_SL0(D(bmask, :), b, sigmaNoise, ...
                    'sigmaMin', sigmaMin);
                
                
            case 'omp'      
                
                % Prepare the (sub)dictionary for omp                                
                Dnorms = sqrt(sum(D(bmask, :).^2) + eps);                                
                                
                ompStopTol = 20;
                x = omp(D(bmask, :) ./ repmat(Dnorms, length(b), 1), ...
                    b, [], ompStopTol);
                
                
                % Watch for normalization constants
                x = x ./ Dnorms(:);                            
                                                       
            
            
        end % switch alg
        
        
        Rec_patch = reshape(D * x + bmean, p, q);
                
        
        
        Srecon(ind1:ind1+p-1, ind2:ind2+q-1)= ...
            Srecon(ind1:ind1+p-1, ind2:ind2+q-1) + ...
            Rec_patch;
        
        Mat_average(ind1:ind1+p-1,ind2:ind2+q-1)= ...
             Mat_average(ind1:ind1+p-1,ind2:ind2+q-1)+ 1;
               

        waitbar(((iter1-1)*l2 + iter2)/(l1*l2), h);
    end % for ind2
    
end % for ind1

close(h)


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Averaging
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Srecon = Srecon ./ Mat_average;


inpainting_time = toc



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% RESULTS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


current_psnr = psnr(S, Srecon, 255)
peak_snr(test, im_nr) = current_psnr;
[ssim, ~] = ssim_index(S, Srecon)
ssimVec(test, im_nr) = ssim;


final_rmse = sqrt(mymse(S(:) - Srecon(:)))

figure(100)
imshow(uint8(Srecon))
title_text = ['Reconstructed image, psnr = ' ...
    num2str(current_psnr) ' dB, ssim ' num2str(ssim)];
title(title_text)

end % for rbr

end % for test

