function show_basis_vectors(W, p, q, varargin)
% Usage: show_basis_vectors(W, p, q, varargin)
%
% Inputs: W - the dictionary matrix
%
%         [p q] - patch size
%
%         optional arguments: size - size of the final image

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[T, N] = size(W); % T = p*q

if p*q ~= T
    error('Wrong patch size')
end

% Check for optional parameters
if(mod(length(varargin),2) == 1)
    error('Optional arguments come in pairs.');
end
for i = 1:2:length(varargin)
    switch lower(varargin{i})
        case 'size'
            size_v = varargin{i+1};        
    end
end

% Dimensions
gap = uint16(q / 4);
if ~exist('size_v', 'var')
    error('Specify size of the matrix')
else
    if numel(size_v) < 2% || prod(size_v) ~= N
        error('Wrong input')
    end
    m = size_v(1);
    n = size_v(2);
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Wmax = max(W(:));

Wmatrix = Wmax * ones(m*p + (m-1) * gap + 2, ...
    n*q + (n-1)*gap + 2);

ind = 1;

for i = 0:m-1
    if ind > N
        break
    end
    for j = 0:n-1
    
       ind1 = 1 + (i*p + 1) + i*gap; ind2 = 1 + (j*q + 1) + j*gap;
       Wmatrix(ind1:ind1+p-1, ind2:ind2+q-1) = ...
           reshape(W(:, ind), p, q);
       ind = ind + 1;        
       if ind > N
           break
       end
       
    end   
end

imagesc(Wmatrix)
colormap gray
axis equal
axis off
